package com.qmth.sop.server.api;

import com.google.gson.reflect.TypeToken;
import com.qmth.sop.business.bean.FormPropertyBean;
import com.qmth.sop.business.bean.result.FlowFormWidgetResult;
import com.qmth.sop.business.service.ActivitiConsumerService;
import com.qmth.sop.common.util.GsonUtil;
import com.qmth.sop.common.util.JacksonUtil;
import io.swagger.annotations.*;
import org.activiti.engine.FormService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.TaskService;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;
import java.util.zip.ZipInputStream;

@Api(tags = "测试流程表单htmlController")
@RestController
@RequestMapping("/activiti/form/html")
public class ActivitiFromHtmlController {
    private final static Logger log = LoggerFactory.getLogger(ActivitiFromHtmlController.class);

    @Resource
    private ActivitiConsumerService activitiConsumerService;

    @Resource
    private RepositoryService repositoryService;

    @Resource
    FormService formService;

    @Resource
    TaskService taskService;

    /**
     * 注册流程
     *
     * @return
     */
    @ApiOperation(value = "创建流程接口")
    @RequestMapping(value = "/createDeployment", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> createDeployment() throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
//        builder.addClasspathResource("testform3.bpmn");
//        builder.addClasspathResource("start.form");
//        builder.addClasspathResource("dept-leader-audit.form");
        ClassPathResource resource = new ClassPathResource("testform3.zip");
//        ClassPathResource resource = new ClassPathResource("testform4.zip");
        ZipInputStream zip = new ZipInputStream(resource.getInputStream());
        builder.addZipInputStream(zip);
        return getFlowStartFormData(builder.deploy().getId());
    }

    /**
     * 获取流程提交表单属性接口
     *
     * @return
     */
    @ApiOperation(value = "获取流程提交表单属性接口")
    @RequestMapping(value = "/getFlowStartFormData", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowStartFormData(@ApiParam(value = "部署流程id", required = true) @RequestParam String deploymentId) {
        Map<String, Object> map = new HashMap<>();
        String processDefinitionId = activitiConsumerService.findProcessDefinitionIdByDeploymentId(deploymentId);
//        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(processDefinitionId).singleResult();
//        boolean hasStartFormKey = processDefinition.hasStartFormKey();
        Object o = formService.getRenderedStartForm(processDefinitionId);
        if (Objects.nonNull(o)) {
            log.info("start formProperties:{}", JacksonUtil.parseJson(o));
            List<FlowFormWidgetResult> list = GsonUtil.fromJson(JacksonUtil.parseJson(o), new TypeToken<List<FlowFormWidgetResult>>() {
            }.getType());
            map.put("FormProperty", new FormPropertyBean(null, null, list));
        }
        map.put("processDefinitionId", processDefinitionId);
        map.put("deploymentId", deploymentId);
        return map;
    }

    /**
     * 获取流程提交表单属性接口
     *
     * @return
     */
    @ApiOperation(value = "获取流程表单属性接口")
    @RequestMapping(value = "/getFlowFormData", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowFormData(@ApiParam(value = "流程id") @RequestParam(required = false) String flowId,
                                               @ApiParam(value = "流程任务id") @RequestParam(required = false) String taskId) {
        List<Task> taskList = null;
        if (Objects.nonNull(flowId) && !Objects.equals(flowId, "")) {
            taskList = taskService.createTaskQuery().processInstanceId(flowId).list();
        } else if (Objects.nonNull(taskId) && !Objects.equals(taskId, "")) {
            taskList = taskService.createTaskQuery().taskId(taskId).list();
        }
        Map<String, Object> varMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String flowTaskId = t.getId();
                String processInstanceId = t.getProcessInstanceId();
                Object o = formService.getRenderedTaskForm(flowTaskId);
//                List<FormProperty> formProperties = taskFormData.getFormProperties();
                log.info("taskId:{},flowId:{}", flowTaskId, processInstanceId);
                if (Objects.nonNull(o)) {
                    log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                    List<FlowFormWidgetResult> list = GsonUtil.fromJson(JacksonUtil.parseJson(o), new TypeToken<List<FlowFormWidgetResult>>() {
                    }.getType());
                    formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, t.getAssignee(), list));
                }
            }
            varMap.put("formProperties", formPropertyList);
        }
        return varMap;
    }

    /**
     * 启动流程
     *
     * @return
     */
    @ApiOperation(value = "启动流程接口")
    @RequestMapping(value = "/startActivityDemo", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> startActivityDemo(@ApiParam(value = "流程定义id", required = true) @RequestParam String processDefinitionId,
                                                 @ApiParam(value = "审批人id", required = true) @RequestParam String approveId,
                                                 @ApiParam(value = "上传文件") @RequestParam(required = false) MultipartFile file,
                                                 HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);

        ProcessInstance processInstance = formService.submitStartFormData(processDefinitionId, formDataMap);
        if (Objects.nonNull(processInstance)) {
            List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstance.getId()).list();
            if (!CollectionUtils.isEmpty(taskList)) {
                List<FormPropertyBean> formPropertyList = new LinkedList<>();
                for (Task t : taskList) {
                    String flowTaskId = t.getId();
                    String processInstanceId = t.getProcessInstanceId();
                    Object o = formService.getRenderedTaskForm(flowTaskId);
                    log.info("flowTaskId:{},flowId:{}", flowTaskId, processInstanceId);
                    if (Objects.nonNull(o)) {
                        log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                        List<FlowFormWidgetResult> list = GsonUtil.fromJson(JacksonUtil.parseJson(o), new TypeToken<List<FlowFormWidgetResult>>() {
                        }.getType());
                        formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, list));
                    }
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    /**
     * 获取待办
     *
     * @return
     */
    @ApiOperation(value = "获取待办接口")
    @RequestMapping(value = "/getTaskList", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getTaskList(@ApiParam(value = "用户id", required = true) @RequestParam String userId, HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        List<Task> taskList = taskService.createTaskQuery().taskAssignee(userId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String flowTaskId = t.getId();
                String processInstanceId = t.getProcessInstanceId();
                String processDefinitionId = t.getProcessDefinitionId();
                log.info("用户id:{},taskId:{},flowId:{},processDefinitionId:{},formKey:{}", userId, flowTaskId, processInstanceId, processDefinitionId, t.getFormKey());
                if (Objects.nonNull(t.getFormKey())) {
                    Object o = formService.getRenderedTaskForm(flowTaskId);
                    if (Objects.nonNull(o)) {
                        log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                        List<FlowFormWidgetResult> list = GsonUtil.fromJson(JacksonUtil.parseJson(o), new TypeToken<List<FlowFormWidgetResult>>() {
                        }.getType());
                        formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, userId, list));
                    }
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    /**
     * 提交
     *
     * @param taskId
     * @return
     */
    @ApiOperation(value = "流程提交接口")
    @RequestMapping(value = "/complete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> complete(@ApiParam(value = "流程taskId", required = true) @RequestParam String taskId,
                                        @ApiParam(value = "上传文件") @RequestParam(required = false) MultipartFile[] files, HttpServletRequest request) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);
        String flowTaskId = task.getId();
        String processInstanceId = task.getProcessInstanceId();
        String processDefinitionId = task.getProcessDefinitionId();
        formService.submitTaskFormData(flowTaskId, formDataMap);
        List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstanceId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String nextFlowTaskId = t.getId();
                Object o = formService.getRenderedTaskForm(nextFlowTaskId);
                log.info("userId:{},taskId:{},flowId:{},processDefinitionId:{}", t.getAssignee(), nextFlowTaskId, processInstanceId, processDefinitionId);
                if (Objects.nonNull(o)) {
                    log.info("task formProperties:{}", JacksonUtil.parseJson(o));
                    List<FlowFormWidgetResult> list = GsonUtil.fromJson(JacksonUtil.parseJson(o), new TypeToken<List<FlowFormWidgetResult>>() {
                    }.getType());
                    formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, t.getAssignee(), list));
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    /**
     * 根据流程id直接结束流程
     *
     * @return
     * @paramru
     */
    @ApiOperation(value = "结束流程接口")
    @RequestMapping(value = "/deleteProcessInstance", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public boolean deleteProcessInstance(String runId) {
        return activitiConsumerService.deleteProcessInstance(runId);
    }
}